@@ -59,16 +59,7 @@ static void PrepareBackbuffer()
 
 boolean ToonNeedsRedraw()
 {
-#if 1
   return TRUE;
-#else
-  return (game_status == GAME_MODE_INFO ||
-	  game_status == GAME_MODE_LEVELS ||
-	  game_status == GAME_MODE_SETUP ||
-	  (game_status == GAME_MODE_MAIN &&
-	   ((redraw_mask & REDRAW_MICROLEVEL) ||
-	    (redraw_mask & REDRAW_MICROLABEL))));
-#endif
 }
 
 void InitToons()
@@ -76,10 +67,10 @@ void InitToons()
   int num_toons = MAX_NUM_TOONS;
   int i;
 
-  if (global.num_toons > 0 && global.num_toons < MAX_NUM_TOONS)
+  if (global.num_toons >= 0 && global.num_toons < MAX_NUM_TOONS)
     num_toons = global.num_toons;
 
-  for (i=0; i < num_toons; i++)
+  for (i = 0; i < num_toons; i++)
   {
     int graphic = IMG_TOON_1 + i;
     struct FileInfo *image = getImageListEntryFromImageID(graphic);
