@@ -21,10 +21,15 @@
 /* functions and definitions exported from main program to game_em           */
 /* ========================================================================= */
 
+#include "game.h"
+
 extern void SetBitmaps_EM(Bitmap **);
 extern void UpdateEngineValues(int, int);
 extern void DrawAllGameValues(int, int, int, int, int);
+
+extern int getNumActivePlayers_EM();
 extern int getGameFrameDelay_EM(int);
+
 extern void PlayLevelSound_EM(int, int, int, int);
 extern void InitGraphicInfo_EM(void);
 
