@@ -16,7 +16,6 @@
 
 #include "main.h"
 
-#if 1
 #define setMoveIntoAcidProperty(l, e, v)			\
 	(setBitfieldProperty(&(l)->can_move_into_acid_bits,	\
 			     EP_CAN_MOVE_INTO_ACID, e, v))
@@ -33,16 +32,12 @@
 void setBitfieldProperty(int *, int, int, boolean);
 boolean getBitfieldProperty(int *, int, int);
 
-#else
-
-void setMoveIntoAcidProperty(struct LevelInfo *, int, boolean);
-boolean getMoveIntoAcidProperty(struct LevelInfo *, int);
-#endif
-
 void InitElementPropertiesStatic(void);
 void InitElementPropertiesEngine(int);
+void InitElementPropertiesAfterLoading(int);
 
 void ReloadCustomArtwork(int);
+void RedrawBackground();
 
 void KeyboardAutoRepeatOffUnlessAutoplay();
 
