@@ -16,6 +16,9 @@
 
 #include "main.h"
 
+#define NETWORK_STOP_BY_PLAYER		0
+#define NETWORK_STOP_BY_ERROR		1
+
 boolean ConnectToServer(char *, int);
 void SendToServer_PlayerName(char *);
 void SendToServer_ProtocolVersion(void);
@@ -23,7 +26,7 @@ void SendToServer_NrWanted(int);
 void SendToServer_StartPlaying(void);
 void SendToServer_PausePlaying(void);
 void SendToServer_ContinuePlaying(void);
-void SendToServer_StopPlaying(void);
+void SendToServer_StopPlaying(int);
 void SendToServer_MovePlayer(byte);
 void HandleNetworking(void);
 
